# GOAL: optimize dose for indiv given its covariates

# loading needed libraries
# library(mlxR)

#=====================================================================================
# plot usual treatment for indiv WT=78 and CLCR=30

# define project file
project.file <- '../monolixProjects/project_07.mlxtran'

# simulate an individual with WT=78 and CLCR=30

weight <- 78
clcr <- 30
outCc  <- list(name = 'Cc', time = seq(279, 295.9, by=0.01))


# define covariates
covariates <- data.frame(id  = 1,
                          WT  = weight,
                          CLCR= clcr,
                          AGE = NaN, #unused but needed for consistency
                          SEX = NaN) #unused but needed for consistency

param=c(omega_k=0,omega_V=0) #disbale random effects to simulate typical individual with this WT and CLCR

# define usual treatment
dosePerKg <- 1
interdoseinterval <- 8
adm <- list(time=seq(0,300,by=interdoseinterval), amount=dosePerKg*weight)

# run simulation of one individual, parameter k is drawn from its distribution, taking into account the covariates
res  <- simulx(project = project.file,
                output  = outCc,
                treatment = adm,
                parameter = list(covariates,param))



d=data.frame(x1=c(-Inf,-Inf), x2=c(Inf,Inf), y1=c(6,0), y2=c(7,2))
p1 <- ggplot() + 
  geom_rect(data=d, mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2),alpha=0.2) +
  geom_line(data=res$Cc, aes(x=time, y=Cc), color='blue', size=1) +
  scale_x_continuous("Time (hr)",  limits=c(279, 295.9)) + 
  scale_y_continuous("Concentration (mg/L)", limits=c(0, 8)) +
  theme(legend.position="none") +
  theme_bw(base_size = 16) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  geom_text(data=res$Cc,x=293.5, y=6.5, label="Target for peak") +
  geom_text(data=res$Cc,x=293.5, y=1.0, label="Target for trough") 
print(p1)
# => the treatment is not good for this individual

#=================================================================================
# find a good dose (efficient and not toxic) for this individual

source('getMinMaxFunction.R')

reference <- NULL
result <- getMinMax(project.file, dosePerKg, interdoseinterval, reference, weight, clcr)

while(result$max < 6.3 | result$max > 6.7 | result$min> 1){
  if (result$max < 6.3 | result$max > 6.7){
    dosePerKg = dosePerKg + 0.2*(6.5-result$max)
  }else if(result$min > 1){
    interdoseinterval = interdoseinterval + 3
  }else{
    paste0("Error")
  }
  result <- getMinMax(project.file, dosePerKg, interdoseinterval,result$ref, weight, clcr)
  
}
cat(paste0("\n","dosePerKg = ",formatC(dosePerKg, digits = 2, format = "f"),"  ;  interdoseinterval = ",interdoseinterval))


#=================================================================================
# check the chances of having a good treatment, given the random effects


#default treatment, applied to tis specific individual: uncomment to run
#interdoseinterval = 8
#dosePerKg = 1

# optimized treatment for this individual:
interdoseinterval = 26
dosePerKg = 1.55

#output
outCc  <- list(name = 'Cc', time = seq(35*interdoseinterval-0.05, 37*interdoseinterval-0.05, by=0.05))

# to get 
N <- 500

# define covariates
covariates <- data.frame(id  = (1:N),
                         WT  = weight,
                         CLCR= clcr,
                         AGE = NaN, #unused but needed for consistency
                         SEX = NaN) #unused but needed for consistency


# define usual treatment
adm <- list(time=seq(0,38*interdoseinterval,by=interdoseinterval), amount=dosePerKg*weight)


# run simulation of one individual, parameter k is drawn from its distribution, taking into account the covariates
res  <- simulx(project = project.file,
               output  = outCc,
               treatment = adm,
               parameter = covariates,
               settings = list(seed=123456789),
               group = list(size=N))


Conc <- res$Cc
Conc$time <- Conc$time - Conc$time[1]

 
 xpoints1 <- c(-1,-1,2*interdoseinterval,2*interdoseinterval)
 ypoints1 <- c(0,2,2,0)
 d1 <- data.frame(xpoints1,ypoints1)
 xpoints2 <- c(-1,-1,2*interdoseinterval,2*interdoseinterval)
 ypoints2 <- c(6,7,7,6)
 d2 <- data.frame(xpoints2,ypoints2)
 p3 <- prctilemlx(Conc) +
  # theme(plot.background=element_rect(fill = "darkgrey")) +
   geom_polygon(data=d1, aes(x=xpoints1, y=ypoints1), alpha=0.3) +
   geom_polygon(data=d2, aes(x=xpoints2, y=ypoints2), alpha=0.3) +
   scale_x_continuous("Time (hr)") + 
   scale_y_continuous("Concentration (mg/L)") 
 

print(p3)


maxes <- tapply(res$Cc$Cc,res$Cc$id,max)
mines <- tapply(res$Cc$Cc,res$Cc$id,min)

# calculate the chance that dosing leads to toxic concentrations 
pToxic <- 100 * length(which(mines>2)) / length(maxes)

#calculate the chance that dosing leads to efficient concentrations
pNotEfficient <- 100 * length(which(maxes<6)) / length(maxes)

cat("Using the optimized dosing:")
cat("\n - There are ",100-pToxic,"% chances that the dosing will be non-toxic. ", sep='')
cat("\n - There are ",100-pNotEfficient,"% chances that the dosing will be effective.", sep='')
