# GOAL: optimize the treatment using the individual parameters
options(warn=-1)

source('getMinMaxFunction_withModel.R')

# define model file
modelfile <- '../monolixProjects/project_indiv_param_estimation/bolus_2cpt_Vkk12k21.txt'

# individual parameters obtained from Monolix
parameters <- data.frame(id = 1,
                         V = 23.8,
                         k = 0.112,
                         k12 = 0.0303,
                         k21 = 0.1)

#output
outCc  <- list(name = 'Cc', time = seq(285, 285+16.9, by=0.01))

# define treatment (optained from optimization with covariates)
interdoseinterval = 26
dosePerKg = 1.55
weight <- 78
adm <- list(time=seq(0,300,by=interdoseinterval), amount=dosePerKg*weight)

# run simulation for this individual
res  <- simulx(model = modelfile,
               output  = outCc,
               treatment = adm,
               parameter = parameters)



d=data.frame(x1=c(-Inf,-Inf), x2=c(Inf,Inf), y1=c(6,0), y2=c(7,2))
p1 <- ggplot() + 
  geom_rect(data=d, mapping=aes(xmin=x1, xmax=x2, ymin=y1, ymax=y2),alpha=0.2) +
  geom_line(data=res$Cc, aes(x=time, y=Cc), color='blue', size=1) +
  scale_x_continuous("Time (hr)",  limits=c(285, 285+16.9)) + 
  scale_y_continuous("Concentration (mg/L)", limits=c(0, 8)) +
  theme(legend.position="none") +
  theme_bw(base_size = 16) +
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank()) +
  geom_text(data=res$Cc,x=285+14.5, y=6.5, label="Target for peak") +
  geom_text(data=res$Cc,x=285+14.5, y=1.0, label="Target for trough") 
print(p1)


# optimization=======================================================
reference <- NULL
result <- getMinMax_withModel(modelfile, dosePerKg, interdoseinterval, reference, weight)

while(result$max < 6.3 | result$max > 6.7 | result$min> 1){
  if (result$max < 6.3 | result$max > 6.7){
    dosePerKg = dosePerKg + 0.2*(6.5-result$max)
  }else if(result$min > 1){
    interdoseinterval = interdoseinterval + 3
  }else{
    paste0("Error")
  }
  result <- getMinMax_withModel(modelfile, dosePerKg, interdoseinterval,result$ref, weight)
  
}
cat(paste0("\n","dosePerKg = ",dosePerKg,"  ;  interdoseinterval = ",interdoseinterval))



#==========================================================================
# simulate the optimized treatment with parameter uncertainty

# optimized treatment:
dosePerKg <- 1.81
interdoseinterval <- 26

weight <- 78
adm <- list(time=seq(0,2000,by=interdoseinterval), amount=dosePerKg*weight)

outCc  <- list(name = 'Cc', time = seq(35*interdoseinterval-0.05, 37*interdoseinterval-0.05, by=0.05))


N <- 100

file.indiv.param <- '../monolixProjects/project_indiv_param_estimation_1000chains/GraphicsData/Covariates/Covariates_Simul.txt'
data <- read.table(file.indiv.param, header = TRUE, stringsAsFactors = FALSE)

V_ind <- exp(data[data$Cov_WT==78,'Param_log.V.'])
k_ind <- exp(data[data$Cov_WT==78,'Param_log.k.'])

# parameter values from distribution
parameters <- data.frame(id  = (1:N),
                         V  = V_ind[1:N],
                         k  = k_ind[1:N],
                         k12 = 0.0303,
                         k21 = 0.1)

# parameter values from distribution
#parameters <- data.frame(id  = (1:N),
#                         V  = rnorm(n=N, mean=23.8, sd=2.26),
#                         k  = rnorm(n=N, mean=0.112, sd=0.024),
#                         k12 = 0.0303,
#                         k21 = 0.1)


# run simulation of N individual, parameter k is drawn from its distribution
outparam <- c("V","k","k12","k21")
res  <- simulx(model = modelfile,
                output  = list(outCc,outparam),
                treatment = adm,
                parameter = parameters)

Conc <- res$Cc
Conc$time <- Conc$time - Conc$time[1]

xpoints1 <- c(-1,-1,2*interdoseinterval,2*interdoseinterval)
ypoints1 <- c(0,2,2,0)
d1 <- data.frame(xpoints1,ypoints1)
xpoints2 <- c(-1,-1,2*interdoseinterval,2*interdoseinterval)
ypoints2 <- c(6,7,7,6)
d2 <- data.frame(xpoints2,ypoints2)
p3 <- prctilemlx(Conc) +
  geom_polygon(data=d1, aes(x=xpoints1, y=ypoints1), alpha=0.3) +
  geom_polygon(data=d2, aes(x=xpoints2, y=ypoints2), alpha=0.3) +
  #theme(plot.background=element_rect(fill = "gray")) +
  scale_x_continuous("Time (hr)") + 
  scale_y_continuous("Concentration (mg/L)") 


print(p3)

maxes <- tapply(res$Cc$Cc,res$Cc$id,max)
mines <- tapply(res$Cc$Cc,res$Cc$id,min)

# calculate the chance that dosing leads to toxic concentrations 
pToxic <- 100 * length(which(mines>2)) / length(maxes)

#calculate the chance that dosing leads to efficient concentrations
pNotEfficient <- 100 * length(which(maxes<6)) / length(maxes)

cat("Using the optimized dosing:")
cat("\n - There are ",100-pToxic,"% chances that the dosing will be safe. ", sep='')
cat("\n - There are ",100-pNotEfficient,"% chances that the dosing will be effective.", sep='')
